@@ -1,8 +1,8 @@
-/* $Header: /usr/people/sam/tiff/tools/RCS/tiffcp.c,v 1.48 1996/01/10 19:35:36 sam Rel $ */
+/* $Header: /cvsroot/osrs/libtiff/tools/tiffcp.c,v 1.3 1999/12/21 17:03:03 mwelles Exp $ */
 
 /*
- * Copyright (c) 1988-1996 Sam Leffler
- * Copyright (c) 1991-1996 Silicon Graphics, Inc.
+ * Copyright (c) 1988-1997 Sam Leffler
+ * Copyright (c) 1991-1997 Silicon Graphics, Inc.
  *
  * Permission to use, copy, modify, distribute, and sell this software and 
  * its documentation for any purpose is hereby granted without fee, provided
@@ -29,6 +29,7 @@
 #include <string.h>
 
 #include <ctype.h>
+#include <assert.h>
 
 #include "tiffio.h"
 
@@ -71,7 +72,7 @@ main(int argc, char* argv[])
 	uint32 deftilewidth = (uint32) -1;
 	uint32 deftilelength = (uint32) -1;
 	uint32 defrowsperstrip = (uint32) -1;
-	uint32 diroff = 0;
+	uint32 diroff = 0, p_diroff = 0;
 	TIFF* in;
 	TIFF* out;
 	char mode[10];
@@ -82,7 +83,7 @@ main(int argc, char* argv[])
 
 	*mp++ = 'w';
 	*mp = '\0';
-	while ((c = getopt(argc, argv, "c:f:l:o:p:r:w:aistBLMC")) != -1)
+	while ((c = getopt(argc, argv, "c:f:l:o:z:p:r:w:aistBLMC")) != -1)
 		switch (c) {
 		case 'a':		/* append to output */
 			mode[0] = 'a';
@@ -109,6 +110,9 @@ main(int argc, char* argv[])
 		case 'o':		/* initial directory offset */
 			diroff = strtoul(optarg, NULL, 0);
 			break;
+		case 'z':		/* initial directory offset */
+			p_diroff = strtoul(optarg, NULL, 0);
+			break;
 		case 'p':		/* planar configuration */
 			if (streq(optarg, "separate"))
 				defconfig = PLANARCONFIG_SEPARATE;
@@ -162,6 +166,12 @@ main(int argc, char* argv[])
 			(void) TIFFClose(out);
 			return (1);
 		}
+		if (p_diroff != 0 && !TIFFSetDirectory(in, p_diroff)) {
+			TIFFError(TIFFFileName(in),
+			    "Error, setting subdirectory at %#x", diroff);
+			(void) TIFFClose(out);
+			return (1);
+		}
 		do {
 			config = defconfig;
 			compression = defcompression;
@@ -175,7 +185,7 @@ main(int argc, char* argv[])
 				(void) TIFFClose(out);
 				return (1);
 			}
-		} while (TIFFReadDirectory(in));
+		} while (TIFFReadDirectory(in) && p_diroff == 0 );
 		(void) TIFFClose(in);
 	}
 	(void) TIFFClose(out);
@@ -205,11 +215,11 @@ processG3Options(char* cp)
 static int
 processCompressOptions(char* opt)
 {
-	if (streq(opt, "none"))
+	if (streq(opt, "none")) {
 		defcompression = COMPRESSION_NONE;
-	else if (streq(opt, "packbits"))
+	} else if (streq(opt, "packbits")) {
 		defcompression = COMPRESSION_PACKBITS;
-	else if (strneq(opt, "jpeg", 4)) {
+	} else if (strneq(opt, "jpeg", 4)) {
 		char* cp = strchr(opt, ':');
 		if (cp && isdigit(cp[1]))
 			quality = atoi(cp+1);
@@ -219,9 +229,9 @@ processCompressOptions(char* opt)
 	} else if (strneq(opt, "g3", 2)) {
 		processG3Options(opt);
 		defcompression = COMPRESSION_CCITTFAX3;
-	} else if (streq(opt, "g4"))
+	} else if (streq(opt, "g4")) {
 		defcompression = COMPRESSION_CCITTFAX4;
-	else if (strneq(opt, "lzw", 3)) {
+	} else if (strneq(opt, "lzw", 3)) {
 		char* cp = strchr(opt, ':');
 		if (cp)
 			defpredictor = atoi(cp+1);
@@ -255,6 +265,7 @@ char* stuff[] = {
 " -f msb2lsb	force msb-to-lsb FillOrder for output",
 "",
 " -c lzw[:opts]	compress output with Lempel-Ziv & Welch encoding",
+"               (no longer supported due to Unisys patent enforcement)", 
 " -c zip[:opts]	compress output with deflate encoding",
 " -c jpeg[:opts]compress output with JPEG encoding",
 " -c packbits	compress output with packbits encoding",
@@ -399,6 +410,7 @@ static struct cpTag {
 	{ TIFFTAG_HALFTONEHINTS,	2, TIFF_SHORT },
 	{ TIFFTAG_INKSET,		1, TIFF_SHORT },
 	{ TIFFTAG_INKNAMES,		1, TIFF_ASCII },
+	{ TIFFTAG_NUMBEROFINKS,		1, TIFF_SHORT },
 	{ TIFFTAG_DOTRANGE,		2, TIFF_SHORT },
 	{ TIFFTAG_TARGETPRINTER,	1, TIFF_ASCII },
 	{ TIFFTAG_SAMPLEFORMAT,		1, TIFF_SHORT },
@@ -409,6 +421,7 @@ static struct cpTag {
 	{ TIFFTAG_EXTRASAMPLES,		(uint16) -1, TIFF_SHORT },
 	{ TIFFTAG_SMINSAMPLEVALUE,	1, TIFF_DOUBLE },
 	{ TIFFTAG_SMAXSAMPLEVALUE,	1, TIFF_DOUBLE },
+	{ TIFFTAG_STONITS,		1, TIFF_DOUBLE },
 };
 #define	NTAGS	(sizeof (tags) / sizeof (tags[0]))
 
@@ -429,19 +442,23 @@ tiffcp(TIFF* in, TIFF* out)
 	CopyField(TIFFTAG_IMAGEWIDTH, w);
 	CopyField(TIFFTAG_IMAGELENGTH, l);
 	CopyField(TIFFTAG_BITSPERSAMPLE, bitspersample);
+	CopyField(TIFFTAG_SAMPLESPERPIXEL, samplesperpixel);
 	if (compression != (uint16)-1)
 		TIFFSetField(out, TIFFTAG_COMPRESSION, compression);
 	else
 		CopyField(TIFFTAG_COMPRESSION, compression);
 	if (compression == COMPRESSION_JPEG && jpegcolormode == JPEGCOLORMODE_RGB)
 		TIFFSetField(out, TIFFTAG_PHOTOMETRIC, PHOTOMETRIC_YCBCR);
+	else if (compression == COMPRESSION_SGILOG || compression == COMPRESSION_SGILOG24)
+		TIFFSetField(out, TIFFTAG_PHOTOMETRIC,
+		    samplesperpixel == 1 ?
+			PHOTOMETRIC_LOGL : PHOTOMETRIC_LOGLUV);
 	else
 		CopyTag(TIFFTAG_PHOTOMETRIC, 1, TIFF_SHORT);
 	if (fillorder != 0)
 		TIFFSetField(out, TIFFTAG_FILLORDER, fillorder);
 	else
 		CopyTag(TIFFTAG_FILLORDER, 1, TIFF_SHORT);
-	CopyField(TIFFTAG_SAMPLESPERPIXEL, samplesperpixel);
 	/*
 	 * Choose tiles/strip for the output image according to
 	 * the command line arguments (-tiles, -strips) and the
@@ -512,6 +529,11 @@ tiffcp(TIFF* in, TIFF* out)
 		CopyTag(TIFFTAG_FAXSUBADDRESS, 1, TIFF_ASCII);
 		break;
 	}
+	{ uint32 len32;
+	  void** data;
+	  if (TIFFGetField(in, TIFFTAG_ICCPROFILE, &len32, &data))
+		TIFFSetField(out, TIFFTAG_ICCPROFILE, len32, data);
+	}
 	for (p = tags; p < &tags[NTAGS]; p++)
 		CopyTag(p->tag, p->count, p->type);
 
@@ -703,12 +725,20 @@ cpStripToTile(uint8* out, uint8* in,
 
 static void
 cpContigBufToSeparateBuf(uint8* out, uint8* in,
-	uint32 rows, uint32 cols, int outskew, int inskew, tsample_t spp)
+           uint32 rows, uint32 cols, int outskew, int inskew, tsample_t spp,
+           int bytes_per_sample )
 {
 	while (rows-- > 0) {
 		uint32 j = cols;
 		while (j-- > 0)
-			*out++ = *in, in += spp;
+                {
+                        int n = bytes_per_sample;
+
+                        while( n-- ) {
+                            *out++ = *in++;
+                        }
+                        in += (spp-1) * bytes_per_sample;
+                }
 		out += outskew;
 		in += inskew;
 	}
@@ -716,12 +746,19 @@ cpContigBufToSeparateBuf(uint8* out, uint8* in,
 
 static void
 cpSeparateBufToContigBuf(uint8* out, uint8* in,
-	uint32 rows, uint32 cols, int outskew, int inskew, tsample_t spp)
+	uint32 rows, uint32 cols, int outskew, int inskew, tsample_t spp,
+                         int bytes_per_sample)
 {
 	while (rows-- > 0) {
 		uint32 j = cols;
-		while (j-- > 0)
-			*out = *in++, out += spp;
+		while (j-- > 0) {
+                        int n = bytes_per_sample;
+
+                        while( n-- ) {
+                                *out++ = *in++;
+                        }
+                        out += (spp-1)*bytes_per_sample;
+                }
 		out += outskew;
 		in += inskew;
 	}
@@ -733,6 +770,7 @@ cpImage(TIFF* in, TIFF* out, readFunc fin, writeFunc fout,
 {
 	int status = FALSE;
 	tdata_t buf = _TIFFmalloc(TIFFRasterScanlineSize(in) * imagelength);
+        
 	if (buf) {
 		(*fin)(in, (uint8*)buf, imagelength, imagewidth, spp);
 		status = (fout)(out, (uint8*)buf, imagelength, imagewidth, spp);
@@ -771,14 +809,16 @@ DECLAREreadFunc(readSeparateStripsIntoBuffer)
 			for (s = 0; s < spp; s++) {
 				uint8* bp = bufp + s;
 				tsize_t n = scanlinesize;
+                                uint8* sbuf = scanline;
 
 				if (TIFFReadScanline(in, scanline, row, s) < 0 && !ignore)
 					goto done;
 				while (n-- > 0)
-					*bp = *bufp++, bp += spp;
+					*bp = *sbuf++, bp += spp;
 			}
-			bufp += scanlinesize;
+			bufp += scanlinesize * spp;
 		}
+
 done:
 		_TIFFfree(scanline);
 	}
@@ -799,6 +839,7 @@ DECLAREreadFunc(readContigTilesIntoBuffer)
 		return;
 	(void) TIFFGetField(in, TIFFTAG_TILEWIDTH, &tw);
 	(void) TIFFGetField(in, TIFFTAG_TILELENGTH, &tl);
+        
 	for (row = 0; row < imagelength; row += tl) {
 		uint32 nrow = (row+tl > imagelength) ? imagelength-row : tl;
 		uint32 colb = 0;
@@ -812,12 +853,12 @@ DECLAREreadFunc(readContigTilesIntoBuffer)
 				uint32 width = imagew - colb;
 				uint32 oskew = tilew - width;
 				cpStripToTile(bufp + colb,
-					tilebuf, nrow, width,
-					oskew + iskew, oskew);
+                                              tilebuf, nrow, width,
+                                              oskew + iskew, oskew );
 			} else
 				cpStripToTile(bufp + colb,
-					tilebuf, nrow, tilew,
-					iskew, 0);
+                                              tilebuf, nrow, tilew,
+                                              iskew, 0);
 			colb += tilew;
 		}
 		bufp += imagew * nrow;
@@ -828,18 +869,23 @@ done:
 
 DECLAREreadFunc(readSeparateTilesIntoBuffer)
 {
-	uint32 imagew = TIFFScanlineSize(in);
+	uint32 imagew = TIFFRasterScanlineSize(in);
 	uint32 tilew = TIFFTileRowSize(in);
-	int iskew  = imagew - tilew;
+	int iskew  = imagew - tilew*spp;
 	tdata_t tilebuf = _TIFFmalloc(TIFFTileSize(in));
 	uint8* bufp = (uint8*) buf;
 	uint32 tw, tl;
 	uint32 row;
+        uint16 bps, bytes_per_sample;
 
 	if (tilebuf == 0)
 		return;
 	(void) TIFFGetField(in, TIFFTAG_TILEWIDTH, &tw);
 	(void) TIFFGetField(in, TIFFTAG_TILELENGTH, &tl);
+	(void) TIFFGetField(in, TIFFTAG_BITSPERSAMPLE, &bps);
+        assert( bps % 8 == 0 );
+        bytes_per_sample = bps/8;
+
 	for (row = 0; row < imagelength; row += tl) {
 		uint32 nrow = (row+tl > imagelength) ? imagelength-row : tl;
 		uint32 colb = 0;
@@ -855,18 +901,24 @@ DECLAREreadFunc(readSeparateTilesIntoBuffer)
 				 * Tile is clipped horizontally.  Calculate
 				 * visible portion and skewing factors.
 				 */
-				if (colb + tilew > imagew) {
+				if (colb + tilew*spp > imagew) {
 					uint32 width = imagew - colb;
-					int oskew = tilew - width;
-					cpSeparateBufToContigBuf(bufp+colb+s,
-					    tilebuf, nrow, width,
-					    oskew + iskew, oskew, spp);
+					int oskew = tilew*spp - width;
+					cpSeparateBufToContigBuf(
+                                            bufp+colb+s*bytes_per_sample,
+					    tilebuf, nrow,
+                                            width/(spp*bytes_per_sample),
+					    oskew + iskew,
+                                            oskew/spp, spp,
+                                            bytes_per_sample);
 				} else
-					cpSeparateBufToContigBuf(bufp+colb+s,
+					cpSeparateBufToContigBuf(
+                                            bufp+colb+s*bytes_per_sample,
 					    tilebuf, nrow, tw,
-					    iskew, 0, spp);
+					    iskew, 0, spp,
+                                            bytes_per_sample);
 			}
-			colb += tilew;
+			colb += tilew*spp;
 		}
 		bufp += imagew * nrow;
 	}
@@ -876,35 +928,44 @@ done:
 
 DECLAREwriteFunc(writeBufferToContigStrips)
 {
-	tsize_t scanline = TIFFScanlineSize(out);
-	uint32 row;
+	uint32 row, rowsperstrip;
+	tstrip_t strip = 0;
 
 	(void) imagewidth; (void) spp;
-	for (row = 0; row < imagelength; row++) {
-		if (TIFFWriteScanline(out, buf, row, 0) < 0)
+	(void) TIFFGetFieldDefaulted(out, TIFFTAG_ROWSPERSTRIP, &rowsperstrip);
+	for (row = 0; row < imagelength; row += rowsperstrip) {
+		uint32 nrows = (row+rowsperstrip > imagelength) ?
+		    imagelength-row : rowsperstrip;
+		tsize_t stripsize = TIFFVStripSize(out, nrows);
+		if (TIFFWriteEncodedStrip(out, strip++, buf, stripsize) < 0)
 			return (FALSE);
-		buf += scanline;
+		buf += stripsize;
 	}
 	return (TRUE);
 }
 
 DECLAREwriteFunc(writeBufferToSeparateStrips)
 {
-	tdata_t obuf = _TIFFmalloc(TIFFScanlineSize(out));
+	uint32 rowsize = imagewidth * spp;
+	uint32 rowsperstrip;
+	tdata_t obuf = _TIFFmalloc(TIFFStripSize(out));
+	tstrip_t strip = 0;
 	tsample_t s;
 
 	if (obuf == NULL)
 		return (0);
+	(void) TIFFGetFieldDefaulted(out, TIFFTAG_ROWSPERSTRIP, &rowsperstrip);
 	for (s = 0; s < spp; s++) {
 		uint32 row;
-		for (row = 0; row < imagelength; row++) {
-			uint8* inp = ((uint8*) buf) + s;
-			uint8* outp = (uint8*) obuf;
-			uint32 n = imagewidth;
+		for (row = 0; row < imagelength; row += rowsperstrip) {
+			uint32 nrows = (row+rowsperstrip > imagelength) ?
+			    imagelength-row : rowsperstrip;
+			tsize_t stripsize = TIFFVStripSize(out, nrows);
 
-			while (n-- > 0)
-				*outp++ = *inp, inp += spp;
-			if (TIFFWriteScanline(out, obuf, row, s) < 0) {
+			cpContigBufToSeparateBuf(
+			    obuf, (uint8*) buf + row*rowsize + s, 
+			    nrows, imagewidth, 0, 0, spp, 1);
+			if (TIFFWriteEncodedStrip(out, strip++, obuf, stripsize) < 0) {
 				_TIFFfree(obuf);
 				return (FALSE);
 			}
@@ -964,16 +1025,22 @@ DECLAREwriteFunc(writeBufferToSeparateTiles)
 {
 	uint32 imagew = TIFFScanlineSize(out);
 	tsize_t tilew  = TIFFTileRowSize(out);
-	int iskew = imagew - tilew;
+	uint32 iimagew = TIFFRasterScanlineSize(out);
+	int iskew = iimagew - tilew*spp;
 	tdata_t obuf = _TIFFmalloc(TIFFTileSize(out));
 	uint8* bufp = (uint8*) buf;
 	uint32 tl, tw;
 	uint32 row;
+        uint16 bps, bytes_per_sample;
 
 	if (obuf == NULL)
 		return (FALSE);
 	(void) TIFFGetField(out, TIFFTAG_TILELENGTH, &tl);
 	(void) TIFFGetField(out, TIFFTAG_TILEWIDTH, &tw);
+	(void) TIFFGetField(out, TIFFTAG_BITSPERSAMPLE, &bps);
+        assert( bps % 8 == 0 );
+        bytes_per_sample = bps/8;
+        
 	for (row = 0; row < imagelength; row += tl) {
 		uint32 nrow = (row+tl > imagelength) ? imagelength-row : tl;
 		uint32 colb = 0;
@@ -987,18 +1054,20 @@ DECLAREwriteFunc(writeBufferToSeparateTiles)
 				 * visible portion and skewing factors.
 				 */
 				if (colb + tilew > imagew) {
-					uint32 width = imagew - colb;
+					uint32 width = (imagew - colb);
 					int oskew = tilew - width;
 
 					cpContigBufToSeparateBuf(obuf,
-					    bufp + colb + s,
-					    nrow, width,
-					    oskew/spp, oskew + imagew, spp);
+					    bufp + (colb*spp) + s,
+					    nrow, width/bytes_per_sample,
+					    oskew, (oskew*spp)+iskew, spp,
+                                            bytes_per_sample);
 				} else
 					cpContigBufToSeparateBuf(obuf,
-					    bufp + colb + s,
+					    bufp + (colb*spp) + s,
 					    nrow, tilewidth,
-					    0, iskew, spp);
+					    0, iskew, spp,
+                                            bytes_per_sample);
 				if (TIFFWriteTile(out, obuf, col, row, 0, s) < 0) {
 					_TIFFfree(obuf);
 					return (FALSE);
@@ -1006,7 +1075,7 @@ DECLAREwriteFunc(writeBufferToSeparateTiles)
 			}
 			colb += tilew;
 		}
-		bufp += nrow * imagew;
+		bufp += nrow * iimagew;
 	}
 	_TIFFfree(obuf);
 	return (TRUE);
